## This Function implements our rapid modified peak finding technique to
## extract all the dominant phynotypic signatures.
# Merger=PeakFinding(x,kst)
# x: Computed phenotypic signature set
# kst: the final number of cellular populations

PeakFinding<-function(x,kst)

{


  temp <-x
  z  <- x


alpha <- 0.2  # Change it inrange 0.05 t0 0.25
k <- alpha * nrow(x)


#  z[i,] - t(x[-i,])   computes the  x1(i) - x2(j)     for a row i  to each other row from 1 to j.
#   rowSums(z[i,] - x[-i,])  is equivalent to colSums(z[i,]- t(x[-i,]))
#    inner lapply   applies j to  the  inner  function(i)
#    outer lapply  passes j from 1 to number of rows in csv file   to the inner lapply function  and calculates mean distance
#     within k nearest neighbours



density <- unlist(
        pblapply(seq_len(nrow(z)), function (j) {
                1/ mean(sort(unlist(lapply(j,function(i){sqrt(colSums((z[i, ] - t(x[-i,]))^2))}))) [1:k] )
        }
        )
)


density <- as.matrix(density)



xs <-   x
zs <-   z


x <- cbind(x,density)
z <- cbind(z,density)


#Max Densed Points
maxdpoints <- which(density == max(density))


# densed distance is the minimum distance of a point k to any other point with higher density
options(warn=-1)
densedistance = pblapply(seq_len(nrow(x)), function(j){
        min(unlist(lapply(which(density[j]<density),function(i){
                sqrt(sum((zs[j,]-xs[i,])^2))
        })))
})
options(warn=0)

dist_maxdpoints = lapply (maxdpoints, function(j) {
        max(unlist( lapply(j, function(i){
                sqrt(colSums((z[i,] - t(x[-i,]))^2))
        })))
}

)



densedistance[maxdpoints]  = dist_maxdpoints

densedistance <- as.matrix(unlist(densedistance))


x <-   cbind (x, densedistance)


fat<-matrix(unlist(x),ncol=4)

mat<-fat[,-c(1:2)]


delta_order<-order(mat[,2])

matlist<-tail(delta_order,kst)

#matlist<-which(mat[,2]>((mean(matlist[,2]))-0.05*sd(mat[,2])))

dominantpeaks<-(temp[matlist,])

kmin<-kmeans(temp,dominantpeaks,1000,nstart=30)

t<-kmin$cluster


return(t)

}
